@@ -23,16 +23,25 @@ discopy computes natural language meaning in pictures.
 >>> assert A(sentence).normal_form() == Alice @ Bob >> love_box
 """
 
-from discopy import cat, monoidal, rigid, tensor, quantum, grammar, biclosed
+
+from discopy import (
+    config, cat, monoidal, rigid, biclosed, tensor, quantum, grammar)
 from discopy.cat import Quiver
+from discopy.monoidal import Sum
 from discopy.rigid import (
     Ob, Ty, PRO, Box, Diagram, Id, Cup, Cap, Swap, Functor)
-from discopy.tensor import Dim, Tensor, TensorFunctor
-from discopy.quantum import (
-    C, Q, CQMap, CQMapFunctor, bit, qubit, Circuit, CircuitFunctor,
-    Discard, MixedState, Measure, Encode, Ket, Bra, Rx, Rz, CRz,
-    SWAP, CZ, CX, H, S, T, X, Y, Z)
-from discopy.grammar import Word
-from discopy.biclosed import Over, Under
+from discopy.tensor import Dim, Tensor, Functor as TensorFunctor
 
-__version__ = '0.3.1'
+from discopy.quantum import cqmap, zx
+from discopy.quantum.zx import Z, X, H, SWAP
+from discopy.quantum.cqmap import C, Q, CQMap
+from discopy.quantum.circuit import (
+    bit, qubit, Circuit, CircuitFunctor, Discard, MixedState, Measure, Encode)
+from discopy.quantum.gates import (
+    Ket, Bra, Bits, Copy, Match, Rx, Rz, CU1, CRz, CRx, CZ, CX)
+
+from discopy.grammar import cfg, ccg, pregroup
+from discopy.grammar.pregroup import Word
+
+
+__version__ = '0.3.4'
