@@ -4,11 +4,6 @@
 discopy error messages.
 """
 
-NUMPY_THRESHOLD = 16
-IGNORE_WARNINGS = [
-    "No GPU/TPU found, falling back to CPU.",
-    "Casting complex values to real discards the imaginary part"]
-
 
 def empty_name(got):
     """ Empty name error. """
@@ -17,9 +12,9 @@ def empty_name(got):
 
 def type_err(expected, got):
     """ Type error. """
-    return "Expected {}.{}, got {} of type {} instead.".format(
+    return "Expected {}.{}, got {} of type {}.{} instead.".format(
         expected.__module__, expected.__name__,
-        repr(got), type(got).__name__)
+        repr(got), type(got).__module__, type(got).__name__)
 
 
 def does_not_compose(left, right):
@@ -41,6 +36,7 @@ def no_winding_number_for_complex_types():
     """ No winding number for complex types. """
     return "Only atomic types have a winding number."
 
+
 def are_not_adjoints(left, right):
     """ Adjunction error. """
     return "{} and {} are not adjoints.".format(left, right)
@@ -75,6 +71,11 @@ def cannot_add(left, right):
     return "Cannot add {} and {}.".format(left, right)
 
 
+def missing_types_for_empty_sum():
+    """ Empty sum needs types. """
+    return "Empty sum needs a domain and codomain."
+
+
 def expected_pregroup():
     """ pregroup.draw error. """
     return "Expected a pregroup diagram of shape `word @ ... @ word >> cups`,"\
