@@ -72,8 +72,8 @@ class Ty(monoidal.Ty, Ob):
     >>> assert (s @ n).l == n.l @ s.l and (s @ n).r == n.r @ s.r
     """
     @staticmethod
-    def upgrade(ty):
-        return Ty(*ty.objects)
+    def upgrade(old):
+        return Ty(*old.objects)
 
     @property
     def l(self):
@@ -112,8 +112,8 @@ class PRO(monoidal.PRO, Ty):
     Objects of the free rigid monoidal category generated by 1.
     """
     @staticmethod
-    def upgrade(ty):
-        return PRO(len(monoidal.PRO.upgrade(ty)))
+    def upgrade(old):
+        return PRO(len(monoidal.PRO.upgrade(old)))
 
     @property
     def l(self):
@@ -127,6 +127,7 @@ class PRO(monoidal.PRO, Ty):
         return self
 
 
+@monoidal.diagram_subclass
 class Diagram(monoidal.Diagram):
     """ Implements diagrams in the free rigid monoidal category.
 
@@ -137,16 +138,8 @@ class Diagram(monoidal.Diagram):
     Alice >> Id(n) @ jokes >> Cup(n, n.r) @ Id(s)
     """
     @staticmethod
-    def upgrade(diagram):
-        """
-        Takes a monoidal.Diagram and returns a rigid.Diagram.
-        """
-        return Diagram(Ty(*diagram.dom), Ty(*diagram.cod),
-                       diagram.boxes, diagram.offsets, layers=diagram.layers)
-
-    @staticmethod
-    def id(x):
-        return Id(x)
+    def id(dom):
+        return Id(dom)
 
     @staticmethod
     def swap(left, right):
@@ -201,13 +194,27 @@ class Diagram(monoidal.Diagram):
         off = -len(right) or len(left)
         return Id(left[:off]) @ Diagram.cups(left[off:], right)
 
-
     @staticmethod
     def ba(left, right):
         """ Backward application. """
         off = -len(left) or len(right)
         return Diagram.cups(left, right[:off]) @ Id(right[off:])
 
+    @staticmethod
+    def fc(left, middle, right):
+        """ Forward composition. """
+        return Id(left) @ Diagram.cups(middle.l, middle) @ Id(right.l)
+
+    @staticmethod
+    def curry(diagram, n_wires=1, left=False):
+        """ Diagram currying. """
+        if left:
+            wires = diagram.dom[:n_wires]
+            return Diagram.caps(wires.r, wires) @ Id(diagram.dom[n_wires:])\
+                >> Id(wires.r) @ diagram
+        wires = diagram.dom[-n_wires or len(diagram.dom):]
+        return Id(diagram.dom[:-n_wires]) @ Diagram.caps(wires, wires.l)\
+            >> diagram @ Id(wires.l)
 
     def transpose(self, left=False):
         """
@@ -222,11 +229,11 @@ class Diagram(monoidal.Diagram):
 
         >>> a, b = Ty('a'), Ty('b')
         >>> double_snake = Id(a @ b).transpose(left=True)
-        >>> two_snakes = Id(b).transpose(left=True) @ Id(a).transpose(left=True)
+        >>> snakes = Id(b).transpose(left=True) @ Id(a).transpose(left=True)
         >>> double_snake == two_snakes
         False
         >>> *_, two_snakes_nf = monoidal.Diagram.normalize(
-        ...     two_snakes, left=True)
+        ...     snakes, left=True)
         >>> assert double_snake == two_snakes_nf
         >>> f = Box('f', a, b)
         """
@@ -355,20 +362,15 @@ class Diagram(monoidal.Diagram):
             normalize=normalize or Diagram.normalize, **params)
 
 
-class Id(Diagram):
+class Id(monoidal.Id, Diagram):
     """ Define an identity arrow in a free rigid category
 
     >>> t = Ty('a', 'b', 'c')
     >>> assert Id(t) == Diagram(t, t, [], [])
     """
-    def __init__(self, t):
-        super().__init__(t, t, [], [], layers=cat.Id(t))
-
-    def __repr__(self):
-        return "Id({})".format(repr(self.dom))
-
-    def __str__(self):
-        return "Id({})".format(str(self.dom))
+    def __init__(self, dom):
+        monoidal.Id.__init__(self, dom)
+        Diagram.__init__(self, dom, dom, [], [], layers=cat.Id(dom))
 
 
 class Box(monoidal.Box, Diagram):
@@ -385,6 +387,10 @@ class Box(monoidal.Box, Diagram):
 
 class Swap(monoidal.Swap, Box):
     """ Implements swaps of basic types in a rigid category. """
+    def __init__(self, left, right):
+        monoidal.Swap.__init__(self, left, right)
+        dom, cod = left @ right, right @ left
+        Box.__init__(self, "Swap({}, {})".format(left, right), dom, cod)
 
 
 class Cup(Box):
@@ -405,17 +411,14 @@ class Cup(Box):
             raise AxiomError(messages.are_not_adjoints(left, right))
         if left == right.r:
             raise AxiomError(messages.wrong_adjunction(left, right, cup=True))
-        self.left, self.right = left, right
-        super().__init__('CUP', left @ right, Ty())
+        self.left, self.right, self.draw_as_wire = left, right, True
+        super().__init__("Cup({}, {})".format(left, right), left @ right, Ty())
 
     def dagger(self):
         return Cap(self.left, self.right)
 
     def __repr__(self):
-        return "Cup({}, {})".format(repr(self.dom[:1]), repr(self.dom[1:]))
-
-    def __str__(self):
-        return "Cup({}, {})".format(self.dom[:1], self.dom[1:])
+        return "Cup({}, {})".format(repr(self.left), repr(self.right))
 
 
 class Cap(Box):
@@ -436,8 +439,8 @@ class Cap(Box):
             raise AxiomError(messages.are_not_adjoints(left, right))
         if left.r == right:
             raise AxiomError(messages.wrong_adjunction(left, right, cup=False))
-        self.left, self.right = left, right
-        super().__init__('CAP', Ty(), left @ right)
+        self.left, self.right, self.draw_as_wire = left, right, True
+        super().__init__("Cap({}, {})".format(left, right), Ty(), left @ right)
 
     def dagger(self):
         return Cup(self.left, self.right)
@@ -445,9 +448,6 @@ class Cap(Box):
     def __repr__(self):
         return "Cap({}, {})".format(repr(self.left), repr(self.right))
 
-    def __str__(self):
-        return "Cap({}, {})".format(self.left, self.right)
-
 
 class Functor(monoidal.Functor):
     """
@@ -470,24 +470,22 @@ class Functor(monoidal.Functor):
 
     def __call__(self, diagram):
         if isinstance(diagram, monoidal.Ty):
-            return sum([self(b) for b in diagram.objects], self.ob_factory())
-        if isinstance(diagram, Ob) and not diagram.z:
-            return self.ob[Ty(diagram.name)]
-        if isinstance(diagram, Ob):
-            result = self(Ob(diagram.name, z=0))
-            if diagram.z < 0:
-                for _ in range(-diagram.z):
-                    result = result.l
-            elif diagram.z > 0:
-                for _ in range(diagram.z):
-                    result = result.r
-            return result
+            def adjoint(obj):
+                result = self.ob[type(diagram)(type(obj)(obj.name, z=0))]
+                if obj.z < 0:
+                    for _ in range(-obj.z):
+                        result = result.l
+                elif obj.z > 0:
+                    for _ in range(obj.z):
+                        result = result.r
+                return result
+            return self.ob_factory().tensor(*map(adjoint, diagram.objects))
         if isinstance(diagram, Cup):
             return self.ar_factory.cups(
-                self(diagram.dom[0]), self(diagram.dom[1]))
+                self(diagram.dom[:1]), self(diagram.dom[1:]))
         if isinstance(diagram, Cap):
             return self.ar_factory.caps(
-                self(diagram.cod[0]), self(diagram.cod[1]))
+                self(diagram.cod[:1]), self(diagram.cod[1:]))
         if isinstance(diagram, monoidal.Diagram):
             return super().__call__(diagram)
         raise TypeError(messages.type_err(Diagram, diagram))
@@ -495,9 +493,9 @@ class Functor(monoidal.Functor):
 
 def cups(left, right, ar_factory=Diagram, cup_factory=Cup, reverse=False):
     """ Constructs a diagram of nested cups. """
-    for ty in left, right:
-        if not isinstance(ty, Ty):
-            raise TypeError(messages.type_err(Ty, ty))
+    for typ in left, right:
+        if not isinstance(typ, Ty):
+            raise TypeError(messages.type_err(Ty, typ))
     if left.r != right and right.r != left:
         raise AxiomError(messages.are_not_adjoints(left, right))
     result = ar_factory.id(left @ right)
