@@ -9,7 +9,7 @@ def test_Dim():
         Dim(-1)
     dim = Dim(2, 3)
     assert Dim(1) @ dim == dim @ Dim(1) == dim
-    assert sum([Dim(1), dim, Dim(4)], Dim(1)) == Dim(2, 3, 4)
+    assert Dim(1).tensor(*(Dim(2, 3), Dim(4), Dim(1))) == Dim(2, 3, 4)
     assert dim[:1] == Dim(3, 2)[1:] == Dim(2)
     assert dim[0] == Dim(3, 2)[1] == 2
     assert repr(Dim(1, 2, 3)) == str(dim) == "Dim(2, 3)"
@@ -24,7 +24,7 @@ def test_Tensor():
         == "Tensor(dom=Dim(2), cod=Dim(2), array=[0, 1, 1, 0])"
     u = Tensor(Dim(2), Dim(2), [1, 0, 0, 0])
     v = Tensor(Dim(2), Dim(2), [0, 0, 0, 1])
-    assert u + v == Id(2)
+    assert u + v == Tensor.id(Dim(2))
     with raises(TypeError):
         u + [0, 0, 0, 1]
     with raises(AxiomError):
@@ -37,7 +37,7 @@ def test_Tensor():
     m = Tensor(Dim(2, 2), Dim(2), arr)
     assert m == m and np.all(m == arr)
     m = Tensor(Dim(2), Dim(2), [0, 1, 1, 0])
-    assert Id(Dim(2)).then(*(m, m)) == m >> m.dagger() == Id(2)
+    assert Tensor.id(Dim(2)).then(*(m, m)) == m >> m.dagger()
 
 
 def test_Tensor_cups():
@@ -59,21 +59,23 @@ def test_Tensor_caps():
     with raises(AxiomError):
         Tensor.caps(Dim(3), Dim(2))
 
+
 def test_Tensor_transpose():
     assert Tensor.caps(Dim(2), Dim(2)).transpose()\
         == Tensor.cups(Dim(2), Dim(2))
 
+
 def test_Tensor_tensor():
-    assert Tensor.id(2) @ Tensor.id(3) == Tensor.id(Dim(2, 3))
+    assert Tensor.id(Dim(2)) @ Tensor.id(Dim(3)) == Tensor.id(Dim(2, 3))
 
     v = Tensor(Dim(1), Dim(2), [1, 0])
     assert v @ v == Tensor(dom=Dim(1), cod=Dim(2, 2), array=[1, 0, 0, 0])
     assert v @ v.dagger() == v << v.dagger()
 
     x, y = Ty('x'), Ty('y')
-    f, g = Box('f', x, x), Box('g', y, y)
+    f, g = rigid.Box('f', x, x), rigid.Box('g', y, y)
     ob, ar = {x: 2, y: 3}, {f: [1, 0, 0, 1], g: list(range(9))}
-    F = TensorFunctor(ob, ar)
+    F = Functor(ob, ar)
     assert F(f) @ F(g) == F(f @ g)
 
 
@@ -83,32 +85,28 @@ def test_tensor_swap():
     swap = Tensor.swap(Dim(2), Dim(3))
     assert f @ g >> swap == swap >> g @ f
 
-    swaps = Tensor.swap(Dim(2), Dim(3, 3))
-    assert swaps == swap @ Id(3) >> Id(3) @ swap
-    assert swaps >> swaps.dagger() == Id(Dim(2, 3, 3))
+
+def test_Functor():
+    assert repr(Functor({Ty('x'): 1}, {})) ==\
+        "tensor.Functor(ob={Ty('x'): 1}, ar={})"
 
 
-def test_TensorFunctor():
-    assert repr(TensorFunctor({Ty('x'): 1}, {})) ==\
-        "TensorFunctor(ob={Ty('x'): 1}, ar={})"
-
-
-def test_TensorFunctor_call():
+def test_Functor_call():
     x, y = Ty('x'), Ty('y')
-    f, g = Box('f', x @ x, y), Box('g', y, Ty())
+    f, g = rigid.Box('f', x @ x, y), rigid.Box('g', y, Ty())
     ob = {x: 2, y: 3}
     ar = {f: list(range(2 * 2 * 3)), g: list(range(3))}
-    F = TensorFunctor(ob, ar)
+    F = Functor(ob, ar)
     assert list(F(f >> g).array.flatten()) == [5.0, 14.0, 23.0, 32.0]
     assert list(F(g.transpose(left=True)).array.flatten()) == [0.0, 1.0, 2.0]
     with raises(TypeError):
         F("Alice")
 
 
-def test_TensorFunctor_swap():
+def test_Functor_swap():
     x, y = Ty('x'), Ty('y')
-    f, g = Box('f', x, x), Box('g', y, y)
-    F = TensorFunctor({x: 2, y: 3}, {f: [1, 2, 3, 4], g: list(range(9))})
+    f, g = rigid.Box('f', x, x), rigid.Box('g', y, y)
+    F = Functor({x: 2, y: 3}, {f: [1, 2, 3, 4], g: list(range(9))})
     assert F(f @ g >> Diagram.swap(x, y)) == F(Diagram.swap(x, y) >> g @ f)
 
 
@@ -117,3 +115,59 @@ def test_AxiomError():
     with raises(AxiomError) as err:
         m >> m
     assert str(err.value) == messages.does_not_compose(m, m)
+
+
+def test_Functor_sum():
+    x, y = Ty('x'), Ty('y')
+    f = rigid.Box('f', x, y)
+    F = Functor({x: 1, y: 2}, {f: [1, 0]})
+    assert F(f + f) == F(f) + F(f)
+
+
+def test_Tensor_radd():
+    m = Tensor(Dim(2, 2), Dim(2), [1, 0, 0, 1, 0, 1, 1, 0])
+    assert 0 + m == m
+
+
+def test_Tensor_iter():
+    v = Tensor(Dim(1), Dim(2), [0, 1])
+    assert list(v) == [0, 1]
+    s = Tensor(Dim(1), Dim(1), [1])
+    assert list(s) == [1]
+
+
+def test_Tensor_subs():
+    from sympy.abc import x
+    s = Tensor(Dim(1), Dim(1), [x])
+    assert s.subs(x, 1) == 1
+
+
+def test_Diagram_cups_and_caps():
+    with raises(AxiomError):
+        Diagram.cups(Dim(2), Dim(3))
+    assert Id(Dim(2)).transpose()\
+        == Frobenius(0, 2, dim=2) @ Id(Dim(2))\
+        >> Id(Dim(2)) @ Frobenius(2, 0, dim=2)
+
+
+def test_Box():
+    f = Box('f', Dim(2), Dim(2), [0, 1, 1, 0])
+    assert repr(f) == "tensor.Box('f', Dim(2), Dim(2), data=[0, 1, 1, 0])"
+    assert f != rigid.Box('f', Dim(2), Dim(2), [0, 1, 1, 0])
+    assert {f: 42}[f] == 42
+
+
+def test_Frobenius():
+    assert repr(Frobenius(1, 2, dim=3)) == "Frobenius(1, 2, dim=3)"
+    assert Frobenius(1, 2, 2).dagger() == Frobenius(2, 1, 2)
+
+
+def test_Swap_to_tn():
+    nodes, order = Swap(Dim(2), Dim(2)).to_tn()
+    assert order == [nodes[0][0], nodes[1][0], nodes[1][1], nodes[0][1]]
+
+
+def test_Tensor_scalar():
+    s = Tensor(Dim(1), Dim(1), [1])
+    for ptype in [int, float, complex]:
+        assert isinstance(ptype(s), ptype)
